@@ -1,12 +1,14 @@
 import asyncio
 import socket
-from zeroconf import Zeroconf, ServiceInfo, ServiceBrowser
+
+from zeroconf import ServiceBrowser, ServiceInfo, Zeroconf
 
 
 class zeroconf:
 
     instance = None
 
+    @staticmethod
     def get_instance(logger):
         if zeroconf.instance is None:
             zeroconf.instance = zeroconf(logger)
