@@ -1,18 +1,18 @@
 import asyncio
-import socket
-import websockets
-import json
-import traceback
-import logging
-import concurrent.futures
 import functools
+import getopt
+import importlib
+import json
+import logging
 import site
 import sys
-import importlib
 import time
-from . import fhem, pkg_installer
-from .core.zeroconf import zeroconf
+import traceback
 
+import websockets
+
+from . import fhem, pkg_installer, utils, version
+from .core.zeroconf import zeroconf
 
 logger = logging.getLogger(__name__)
 
@@ -48,13 +48,14 @@ async def activate_internal_modules():
 
 
 async def pybinding(websocket, path):
-    if len(sys.argv) == 1 and zc_info is not None:
+    if zc_info is not None:
         # FHEM discovered us, stop zeroconf
-        zeroconf.get_instance(logger).unregister_service(zc_info)
+        await zeroconf.get_instance(logger).unregister_service(zc_info)
+        zeroconf.get_instance(logger).stop()
 
     global connection_start
     connection_start = time.time()
-    logger.info("FHEM connection started: " + websocket.remote_address[0])
+    logger.info("Incoming FHEM connection: " + websocket.remote_address[0])
     pb = PyBinding(websocket)
     fhem.updateConnection(pb)
     await activate_internal_modules()
@@ -123,9 +124,9 @@ class PyBinding:
     async def onMessage(self, payload):
         try:
             await self._onMessage(payload)
-        except SystemExit as se:
+        except SystemExit:
             sys.exit(1)
-        except:
+        except Exception:
             logger.exception("Failed to handle message: " + str(payload))
 
     async def _onMessage(self, payload):
@@ -134,7 +135,7 @@ class PyBinding:
         hash = None
         try:
             hash = json.loads(msg)
-        except:
+        except Exception:
             logger.error("Websocket JSON couldn't be decoded")
             return
 
@@ -154,7 +155,16 @@ class PyBinding:
             else:
                 ret = ""
                 if hash["msgtype"] == "update":
+                    await fhem.readingsSingleUpdate(
+                        hash, "version", "update started...", 1
+                    )
                     await pkg_installer.force_update_package("fhempy")
+                    await fhem.readingsSingleUpdate(
+                        hash,
+                        "version",
+                        "update finished...please wait",
+                        1,
+                    )
                     sys.exit(1)
                 if hash["msgtype"] == "function":
                     # this is needed to avoid 2 replies on dep installation
@@ -168,7 +178,7 @@ class PyBinding:
                                 hash["args"][1]
                             ] = loadedModuleInstances[hash["args"][0]]
                             del loadedModuleInstances[hash["args"][0]]
-                            self.sendBackReturn(hash, "")
+                            await self.sendBackReturn(hash, "")
                             return 0
 
                     if hash["function"] != "Undefine":
@@ -224,11 +234,11 @@ class PyBinding:
                                     await fhem.readingsSingleUpdate(
                                         hash,
                                         "state",
-                                        "Installation finished. Define now...",
+                                        "Installation finished. Please wait...",
                                         1,
                                     )
-                                    # wait 5s so that user can read the msg about installation
-                                    await asyncio.sleep(5)
+                                    # wait 3s so that user can read the msg about installation
+                                    await asyncio.sleep(3)
                                     # continue define
 
                                 # import module
@@ -238,7 +248,10 @@ class PyBinding:
                                     + "."
                                     + hash["PYTHONTYPE"]
                                 )
-                                module_object = importlib.import_module(pymodule)
+                                # import might take a long time, therefore run_blocking
+                                module_object = await utils.run_blocking(
+                                    functools.partial(importlib.import_module, pymodule)
+                                )
                                 # create instance of class with logger
                                 target_class = getattr(
                                     module_object, hash["PYTHONTYPE"]
@@ -299,7 +312,7 @@ class PyBinding:
 
                     nmInstance = loadedModuleInstances[hash["NAME"]]
 
-                    if nmInstance != None:
+                    if nmInstance is not None:
                         try:
                             # handle verbose level of logging
                             if (
@@ -327,7 +340,7 @@ class PyBinding:
                                             ret = await asyncio.wait_for(
                                                 func(hash), fct_timeout
                                             )
-                                        except:
+                                        except Exception:
                                             del loadedModuleInstances[hash["NAME"]]
                                     else:
                                         ret = await asyncio.wait_for(
@@ -350,7 +363,7 @@ class PyBinding:
                             else:
                                 await self.sendBackError(hash, errorMsg)
                             return 0
-                        except:
+                        except Exception:
                             errorMsg = (
                                 "Failed to execute function "
                                 + hash["function"]
@@ -378,30 +391,78 @@ class PyBinding:
             logger.error("Failed to handle message: ", exc_info=True)
 
 
+def usage():
+    print("Usage: fhempy [-h|--help] [-v|--version] [-i|--ip] [-p|--port] [-l|--local]")
+    print("  --local   Use only if you run fhempy on your FHEM machine")
+    print(
+        "  --ip      "
+        "Specify the IP address for FHEM connection setup (default: local ip)"
+    )
+    print("  --port    Specify the port fhempy runs on (default: 15733)")
+    print("  --version Print version and exit")
+    print("  --help    This help text")
+
+
 def run():
-    logger.info("Starting pythonbinding...")
+    try:
+        opts, args = getopt.getopt(
+            sys.argv[1:],
+            "dhvli:p:",
+            ["help", "version", "ip=", "port=", "local", "debug"],
+        )
+    except getopt.GetoptError as err:
+        logger.error(err)
+        usage()
+        sys.exit(2)
+
+    ip = None
+    port = 15733
+    local = False
+    for o, a in opts:
+        if o in ("-v", "--version"):
+            print("fhempy " + str(version.__version__))
+            sys.exit()
+        elif o in ("-h", "--help"):
+            usage()
+            sys.exit()
+        elif o in ("-i", "--ip"):
+            ip = a
+        elif o in ("-p", "--port"):
+            port = a
+        elif o in ("-l", "--local"):
+            local = True
+        elif o in ("-d", "--debug"):
+            logging.getLogger("").setLevel(logging.DEBUG)
+
+    logger.info("Starting fhempy...")
 
     asyncio.get_event_loop().run_until_complete(
         pkg_installer.check_and_install_dependencies("core")
     )
 
-    if len(sys.argv) == 1:
+    if not local:
+        logger.info("Advertise fhempy on local network")
         # running on remote peer, start zeroconf for autodiscovery
-        hostname = socket.gethostname()
-        local_ip = socket.gethostbyname(hostname)
+        if ip is None:
+            local_ip = utils.get_local_ip()
+        else:
+            local_ip = ip
         zc = zeroconf.get_instance(logger)
+        global zc_info
         zc_info = asyncio.get_event_loop().run_until_complete(
             zc.register_service(
                 "_http",
                 "fhempy (" + local_ip + ")",
-                15733,
-                {"port": 15733, "ip": local_ip},
+                port,
+                {"port": port, "ip": local_ip},
             )
         )
 
+    logger.info("Waiting for FHEM connection")
+    asyncio.get_event_loop().set_debug(True)
     asyncio.get_event_loop().run_until_complete(
         websockets.serve(
-            pybinding, "0.0.0.0", 15733, ping_timeout=None, ping_interval=None
+            pybinding, "0.0.0.0", port, ping_timeout=None, ping_interval=None
         )
     )
     asyncio.get_event_loop().run_forever()
